<?php

namespace Drupal\block_condition_published\Plugin\Condition;

use Drupal\Component\Plugin\Exception\ContextException;
use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Published State' condition.
 *
 * @Condition(
 *    id = "published_state",
 *    label = @Translation("Published state"),
 *    deriver = "Drupal\block_condition_published\Plugin\Condition\Deriver\PublishedStateDeriver"
 *  )
 */
class PublishedState extends ConditionPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Creates a new IsPublishedCondition instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $default_config = [
      'is_published' => 0,
      'valid_context_required' => 0,
    ];

    return $default_config + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $form['is_published'] = [
      '#title' => $this->t('Published'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['is_published'],
    ];

    $build = parent::buildConfigurationForm($form, $form_state);
    unset($build['context_mapping']);

    $build['advanced'] = [
      '#type' => 'details',
      '#open' => FALSE,
      '#title' => $this->t('Advanced settings'),
    ];

    $build['advanced']['valid_context_required'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Require a valid context'),
      '#description' => $this->t('If checked, the block will be hidden in context of a non-publishable entity (e.g. user)'),
      '#default_value' => $this->configuration['valid_context_required'],
    ];

    if ($this->moduleHandler->moduleExists('content_moderation')) {
      $build['advanced']['always_use_latest_revision'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Use status of the latest revision'),
        '#description' => $this->t('If enabled, the publishing state of the latest revision will always be used (`Draft`, `Archived` are treated as unpublished), even though the content has a published version.<br>
          Useful only if <a href="@url" target="_blank" rel="external">Content Moderation</a> states are in use.',
          ['@url' => 'https://www.drupal.org/docs/8/core/modules/content-moderation/overview']
        ),
        '#default_value' => $this->configuration['always_use_latest_revision'],
      ];
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['is_published'] = $form_state->getValue('is_published');
    $this->configuration['valid_context_required'] = $form_state->getValue(['advanced', 'valid_context_required']);

    if ($this->moduleHandler->moduleExists('content_moderation')) {
      $this->configuration['always_use_latest_revision'] = $form_state->getValue(
        ['advanced', 'always_use_latest_revision']
      );
    }

    // Always use route context.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $context_mapping = [];
    foreach ($this->getContextDefinitions() as $context_slot => $definition) {
      $valid_contexts = $this->contextHandler()->getMatchingContexts($contexts, $definition);
      // Route context id e.g. "@node.node_route_context:node".
      $route_context_id = sprintf('@%1$s.%1$s_route_context:%1$s', $context_slot);
      if (isset($valid_contexts[$route_context_id])) {
        $context_mapping[$context_slot] = $route_context_id;
      }
    }
    $this->setContextMapping($context_mapping);
  }

  /**
   * {@inheritdoc}
   */
  public function summary() {
    $status = $this->configuration['is_published'];

    if (!empty($this->configuration['negate'])) {
      return $this->t('The published state is not @state', ['@state' => $status]);
    }
    return $this->t('The published state is @state', ['@state' => $status]);
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate() {
    if (empty($this->configuration['is_published']) && !$this->isNegated()) {
      // Ignore if all checkboxes unchecked.
      return TRUE;
    }

    /** @var \Drupal\Core\Entity\EntityPublishedInterface $entity */
    $context = array_filter($this->getContextValues(), fn($entity) => is_object($entity));
    $context_slots = array_keys($context);
    $context_slot = reset($context_slots);

    try {
      $entity = $this->getContextValue($context_slot);
    }
    catch (ContextException $e) {
      // If no context available.
      return $this->configuration['valid_context_required']
        ? $this->isNegated()
        : !$this->isNegated();
    }

    if (!$entity->getEntityType()->hasKey('published') || $entity->isNew()) {
      // If no valid context available.
      return $this->configuration['valid_context_required']
        ? $this->isNegated()
        : !$this->isNegated();
    }

    if ($this->configuration['always_use_latest_revision']) {
      /** @var \Drupal\Core\Entity\EntityStorageInterface $storage */
      $storage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());

      $vid_latest = $storage->getLatestRevisionId($entity->id());

      /** @var \Drupal\Core\Entity\RevisionableInterface $latest_revision */
      $latest_revision = $storage->loadRevision($vid_latest);

      // More info about revisions:
      // https://www.phase2technology.com/blog/getting-correct-node
      // https://drupal.stackexchange.com/questions/264674/what-is-the-default-revision
      return $latest_revision->get('status')->value == $this->configuration['is_published'];
    }

    return $entity->isPublished() == $this->configuration['is_published'];
  }

}
